/*
===============================================================================
 Name        : main.c
 Author      : 
 Version     :
 Copyright   : Copyright (C) 
 Description : main definition
===============================================================================
*/

#include "LPC17xx.h"
#include "lpc17xx_ssp.h"
#include "lpc17xx_rtc.h"
#include <cr_section_macros.h>
#include <NXP/crp.h>
#include "define.h"
#include "types.h"
#include "lcd_eadog102.h"
#include "font_6x8.h"
#include "font_8x16.h"
#include "system.h"
#include "am2302.h"

// Variable to store CRP value in. Will be placed automatically
// by the linker when "Enable Code Read Protect" selected.
// See crp.h header for more information
__CRP const unsigned int CRP_WORD = CRP_NO_CRP ;

volatile uint32_t msTicks; // counter for 1ms SysTicks
volatile uint32_t secondTick;		// ticks from the RTC

//extern volatile struct dcf_format dcf;

RTC_TIME_Type time;			// internal time
struct am2303_data sensor_data;


// ****************
//  SysTick_Handler - just increment SysTick counter
void SysTick_Handler(void) {
  msTicks++;
}

// ****************
// ms_delay - creates a delay of the appropriate number of Systicks (happens every 1 ms)
void ms_delay (uint32_t delayTicks) {
  uint32_t currentTicks;

  currentTicks = msTicks;	// read current tick counter
  // Now loop until required number of ticks passes.
  while ((msTicks - currentTicks) < delayTicks);
}

/*----------------------------------------------------------------------*/
//! Setup the GPIOs
/*----------------------------------------------------------------------*/
void gpioInit()
{
	//Setup the io-pins for the spi-lcd port
	LPC_PINCON->PINSEL0 |= 0x2<<14; //SCK1
	LPC_PINCON->PINSEL0 |= 0x2<<18;	//MOSI1
	LPC_PINCON->PINSEL0 |= 0x2<<16;	// MISO
	PORT_LCD->FIODIR 	|= PIN_MASK_LCDRST;	//P0.1 LCD Reset
	PORT_LCD->FIODIR	|= PIN_MASK_LCDCS;	//P0.18 LCD CS
	PORT_LCD->FIODIR 	|= PIN_MASK_LCDA0;	//P0.6 LCD A0

	// Set LCD IO-Pins High
	PORT_LCD->FIOSET	|= PIN_MASK_LCDCS | PIN_MASK_LCDRST;


}

/*----------------------------------------------------------------------*/
//! Setup the SSP1 for the LCD
/*----------------------------------------------------------------------*/
void initSSP()
{
	SSP_CFG_Type sspChannelConfig;

	sspChannelConfig.CPHA = SSP_CPHA_SECOND;
	sspChannelConfig.CPOL = SSP_CPOL_LO;
	sspChannelConfig.ClockRate = 1000000;
	sspChannelConfig.Databit = SSP_DATABIT_8;
	sspChannelConfig.Mode = SSP_MASTER_MODE;
	sspChannelConfig.FrameFormat = SSP_FRAME_SPI;

	SSP_Init(SSP_CHANNEL, &sspChannelConfig);
	SSP_Cmd(SSP_CHANNEL, ENABLE);
}

/*----------------- INTERRUPT SERVICE ROUTINES --------------------------*/
/*********************************************************************//**
 * @brief		RTC interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void RTC_IRQHandler(void)
{
	uint32_t secval;

	/* This is increment counter interrupt*/
	if (RTC_GetIntPending(LPC_RTC, RTC_INT_COUNTER_INCREASE))
	{
		secondTick ++;

		// Clear pending interrupt
		RTC_ClearIntPending(LPC_RTC, RTC_INT_COUNTER_INCREASE);
	}
}

/*----------------------------------------------------------------------*/
//!< Setup of the RTC
/*----------------------------------------------------------------------*/
void rtcInit()
{
	/* Disable RTC interrupt */
	NVIC_DisableIRQ(RTC_IRQn);
	/* preemption = 1, sub-priority = 1 */
	NVIC_SetPriority(RTC_IRQn, ((0x01<<3)|0x01));

	/* Enable rtc (starts increase the tick counter and second counter register) */
	RTC_ResetClockTickCounter(LPC_RTC);
	RTC_Cmd(LPC_RTC, ENABLE);
	RTC_CalibCounterCmd(LPC_RTC, DISABLE);

	/* Set current time for RTC */
	// Current time is 8:00:00PM, 2009-04-24
//	RTC_SetTime (LPC_RTC, RTC_TIMETYPE_SECOND, 0);
//	RTC_SetTime (LPC_RTC, RTC_TIMETYPE_MINUTE, 0);
//	RTC_SetTime (LPC_RTC, RTC_TIMETYPE_HOUR, 22);
//	RTC_SetTime (LPC_RTC, RTC_TIMETYPE_MONTH, 8);
//	RTC_SetTime (LPC_RTC, RTC_TIMETYPE_YEAR, 2012);
//	RTC_SetTime (LPC_RTC, RTC_TIMETYPE_DAYOFMONTH, 12);

	RTC_CntIncrIntConfig (LPC_RTC, RTC_TIMETYPE_SECOND, ENABLE);

	secondTick = 0;

    /* Enable RTC interrupt */
    NVIC_EnableIRQ(RTC_IRQn);
}

/*----------------------------------------------------------------------*/
//! Get the real the from the RTC and write it to LCD
/*----------------------------------------------------------------------*/
void time2lcd()
{
	uint8_t	text_buffer[20];

	RTC_GetFullTime (LPC_RTC, &time);

	sprintf(text_buffer,"%02d.%02d.%02d %02d:%02d:%02d",time.DOM,time.MONTH,time.YEAR-2000,time.HOUR,time.MIN,time.SEC);
	lcdString(0,0,text_buffer, ptr_font_6x8, DELETE);
	lcdLine(0, 10, 101, 10);
}

/*----------------------------------------------------------------------*/
//! Display the sensor data
//! @param	temperatur	Temperatur [Unit 0.1°C]
//! @param 	humidity	Humidity [Unit 0.1%]
/*----------------------------------------------------------------------*/
void display_sensordata(uint32_t	temperatur, uint32_t humidity)
{
	uint8_t	text_buffer[20];

	sprintf(text_buffer,"%2d.%d \370C %3d.%d %%",temperatur/10,temperatur%10, humidity/10,humidity%10);
	lcdString(0,20,text_buffer, ptr_font_6x8, DELETE);
}



/*----------------------------------------------------------------------*/
//! BEWARE OF THE PROPELLER -> THE MAIN LOOP IS RUNNING
/*----------------------------------------------------------------------*/
int main(void)
{
	uint8_t 	text_buffer[20];
	uint32_t	count;

	//Setup Portpins
	gpioInit();
	rtcInit();

	initSSP();

	// TODO: insert code here
	// Setup SysTick Timer to interrupt at 1 msec intervals
	if (SysTick_Config(SystemCoreClock / 1000)) {
	    while (1);  // Capture error
	}

	lcdInit();
	am2303Init();


	// Enter an infinite loop, just incrementing a counter
	volatile static int i = 0 ;
	while(1) {

		if(secondTick != 0)
		{
			secondTick = 0;
			count++;
			time2lcd();

			if(count> 2)		// Read the T/H-Sensor all 2 seconds
			{
				count = 0;
				if(am2302Read(& sensor_data) == TRUE)
					display_sensordata( sensor_data.temperatur, sensor_data.humidity);
			}
		}

		i++ ;
	}
	return 0 ;
}


